<?php

declare(strict_types=1);

namespace PHPUnit\Architecture\Elements;

use PHPUnit\Architecture\Asserts\Properties\ObjectPropertiesDescription;

class ObjectDescription extends ObjectPropertiesDescription
{
}
